#include<18f458.h>
#include<math.h>

#define	DISTANCE 384	// distance from pointer to display in 1/16"
#define D_WIDTH 144		// width of display
#define D_HEIGHT 88
#define STEPS_X 2000	// horizontal motor steps per 360 degrees
#define STEPS_Y 2000	// vertical
#define STEPDELAY 25

#use delay( clock=10000000 )
#use rs232( baud=9600, xmit=PIN_C6, rcv=PIN_C7 )

int16	curX = 0,	// current total number of steps rotated right
		curY = 0;	// up
unsigned char output = 0x11;

unsigned char bitsForStepX( int n )
{
	switch( n % 4 ) {
		case 0:	return 8;
		case 1:	return 2;
		case 2:	return 4;
		case 3:	return 1;
	}
}

unsigned char bitsForStepY( int n )
{
	switch( n % 4 ) {
		case 0:	return 1;
		case 1:	return 4;
		case 2:	return 2;
		case 3:	return 8;
	}
}

int32 point( signed int16 x, signed int16 y )
{
	float tmp;
	signed int dirX = 1, dirY = 1;
	unsigned int32 clocks = 0;

	set_timer0(0);

	// convert absolute 1/16" x/y to relative steps
//	printf("(%ld,%ld): (%ld,%ld)", curX, curY, x, y);
	x = atan( (double)x / DISTANCE ) * STEPS_X / (2*PI);
	y = atan( (double)y / DISTANCE ) * STEPS_Y / (2*PI);
//	printf("->(%ld,%ld)", x, y);
	x -= curX;
	y -= curY;
//	printf("->(%ld,%ld)\n\r", x, y);

	// pretty up x/y for loop
	if( x < 0 ) {
		dirX = -1;
		x = -x;
	}
	if( y < 0 ) {
		dirY = -1;
		y = -y;
	}

	clocks += get_timer0();

	// do the movement
	while( x>0 || y>0 ) {
		set_timer0(0);
//		printf("%ld:%ld:%d:%d:%ld:%ld\n\r", x,y,dirX,dirY,curX,curY);
		if( x /*&& (dirX>0 || curX>0)*/ ) {
			--x;
			output &= 0xf0;
			output |= bitsForStepX( (curX+=dirX) );
		}
		if( y /*&& (dirY>0 || curY>0)*/ ) {
			--y;
			output &= 0xf;
			output |= bitsForStepY( (curY+=dirY) ) << 4;
		}
		output_d( output );
		delay_ms( STEPDELAY );
		clocks += get_timer0();
	}

	return (double)clocks / 9.765;
//	return (double)clocks / 1249.92;
}

void showSlide( int which )
{
	printf( "%1lX", which );
	delay_ms( 200 );
	putc( 'G'+which );
}

void wait( signed int32 ms )
{
	int16 loops;
	static signed int32 residual = 0;

//	printf( "wait(%Ld)", ms );

	if( residual ) {
		ms += residual;
		residual = 0;
	}

	if( ms < 0 ) {
		residual = ms;
		return;
	}

	loops = ms / 255;
	ms = ms % 255;
	while( loops-- > 0 ) delay_ms(255);
	delay_ms(ms);
}

void main( void )
{
	int16 i = 0;

//	setup_timer_0( RTCC_INTERNAL | RTCC_DIV_256 );
	setup_timer_0( RTCC_INTERNAL | RTCC_DIV_256 );
	output_d( output );

	/*
	char ch = '0';
	output_d(0);
	while(1) {
		output_d(0xff);
		delay_ms(1000);
		output_d(0);
		delay_ms(1000);
		putc(ch++);
	}
	*/
//	while(1) {
//		point( 0, 112 );
//		point( 0, 0 );
		/*
		for( i=0; i<450; ++i ) {
			output_d( (bitsForStep(i)<<4) + 1 );
			delay_ms(50);
		}
		for( ; i>0; --i ) {
			output_d( (bitsForStep(i)<<4) + 1 );
			delay_ms(50);
		}
		*/
//	}
/*
while(1) {
	i = 0;
	showSlide( i++ );
	delay_ms(14000);

	showSlide( i++ );
	delay_ms(23000);

	showSlide( i++ );
	delay_ms(36000);

	showSlide( i++ );
	delay_ms(82000);

	showSlide( i++ );
	delay_ms(16000);

	showSlide( i++ );
	delay_ms(34000);

	showSlide( i++ );
	delay_ms(21000);

	showSlide( i++ );
	delay_ms(24000);

	showSlide( i++ );
	delay_ms(8000);

	showSlide( i++ );
	delay_ms(46000);

	showSlide( i++ );
	delay_ms(12000);
}
*/
/*
	while(1) {
		set_timer0(0);
		delay_ms(100);	// 4841
		printf( " %ld ", get_timer0() );
		set_timer0(0);
		delay_ms(1000);
		printf( " %ld ", get_timer0() );
	}
*/

	// calibration movement
	point(0,0);
	delay_ms(10000);
	point( D_WIDTH, D_HEIGHT );
	delay_ms(5000);
	point(0,0);
	delay_ms(10000);

	// first slide/14
	showSlide( i++ );
	wait(4000);
	wait( 4000 - point(96, 76) );
	wait( 1000 - point(96, 36) );
	wait( 2000 - point(96, 30) );
	wait( 3000 - point(96, 46) );
	point(0,0);

	// second slide/23
	showSlide( i++ );
	wait(9000);
	wait( 6000 - point(16, 80) );
	wait( 8000 - point(44, 80) );
	point(0,0);

	// slide 3/36
	showSlide( i++ );
	wait(7000);
	wait( 5000 - point(72, 60) );
	wait( 3000 - point(72, 52) );
	wait( 3000 - point(88, 42) );
	wait( 2000 - point(72, 36) );
	wait( 2000 - point(72, 31) );
	wait( 4000 - point(88, 22) );
	wait( 4000 - point(72, 18) );
	wait( 6000 - point(72, 10) );
	point(0,0);

	// slide 4/82
	showSlide( i++ );
	wait(8000);
	wait( 11000 - point(66, 54) );
	wait( 3000 - point(66, 38) );
	wait( 12000 - point(66, 15) );
	wait( 4000 - point(96, 72) );
	wait( 7000 - point(96, 60) );
	wait( 11000 - point(96, 49) );
	wait( 4000 - point(128, 65) );
	wait( 4000 - point(128, 51) );
	wait( 18000 - point(128, 35) );
	point(0,0);

	// slide 5/16
	showSlide( i++ );
	wait(5000);
	wait( 11000 - point(40, 68) );
	point(0,0);

	// slide 6/34
	showSlide( i++ );
	wait(14000);
	wait( 6000 - point(4, 58) );
	wait( 14000 - point(4, 36) );
	point(0,0);

	// slide 7/21
	showSlide( i++ );
	wait( 11000 - point(8, 64) );
	wait( 10000 - point(8, 24) );
	point(0,0);

	// slide 8/24
	showSlide( i++ );
	wait( 15000 - point(2, 56) );
	wait( 9000 - point(2, 34) );
	point(0,0);

	// slide 9/8
	showSlide( i++ );
	wait( 8000 - point(70, 46) );
	point(0,0);

	// slide 10/46
	showSlide( i++ );
	wait(9000);
	wait( 5000 - point(8, 76) );
	wait( 6000 - point(4, 60) );
	wait( 10000 - point(4, 48) );
	wait( 16000 - point(4, 24) );
	point(0,0);

	// slide 11/12
	showSlide( i++ );
	wait(3000);
	wait( 6000 - point(40, 72) );
	wait( 3000 - point(40, 12) );
	point(0,0);
}
